using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using System.Xml.Serialization;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebPartPages;
using System.Threading;
using System.Net;
using System.Data;
using Microsoft.SharePoint.WebControls;
using System.IO;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Collections;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls.WebParts;
using System.Reflection;

namespace SPAjaxDemo
{
    [XmlRoot(Namespace = "SPAjaxDemo.AjaxWebpart")]
    public class AjaxWebpart : Microsoft.SharePoint.WebPartPages.WebPart, System.Web.UI.ICallbackEventHandler
    {
        private string _DataDiv; //This will hold the name of your div tag
        private string _AjaxData; //This will hold the data that is returned via ajax...
        private string _ErrorText; //will hold error.ToString() messages
        private string _GroupField;
        private string _SortString;
        private string _FilterString;
        private ArrayList _DisplayFields; // takes the fields, which shoul be displayed
        private SPGridView _Grid;
        private string GridClientID;
        private DataSet _DS;
        private HtmlGenericControl _Div;
        private int _CurrentPage;
        private Label _LblCurrentPage;

        #region Properties
        [WebPartStorage(Storage.None)]
        private string SortExpression
        {
            get
            {
                if (ViewState["SortExpression"] != null)
                    return ViewState["SortExpression"].ToString();
                else
                    return "";
            }
            set
            {

                ViewState["SortExpression"] = value;
            }
        }

        [WebPartStorage(Storage.None)]
        private string SortDirection
        {
            get
            {
                System.Web.UI.WebControls.SortDirection sort = System.Web.UI.WebControls.SortDirection.Ascending;

                if (ViewState["SortDirection"] != null)
                {
                    return ViewState["SortDirection"].ToString();
                }
                else
                {
                    return "";
                }
            }
            set
            {
                ViewState["SortDirection"] = value;
            }
        }

        [Browsable(true),
        Category("Properties"),
        WebPartStorage(Storage.Shared),
        FriendlyName("Groupfield"),
        Description("If desired, fill in the Groupfield (Name)")]
        public string GroupField
        {
            get { return _GroupField; }
            set { _GroupField = value; }
        }

        [Browsable(true),
        Category("Properties"),
        WebPartStorage(Storage.Shared),
        FriendlyName("Sort String"),
        Description("Specify the initial sortstring (Name;ASC)")]
        public string SortString
        {
            get { return _SortString; }
            set { _SortString = value; }
        }

        [Browsable(true),
        Category("Properties"),
        WebPartStorage(Storage.Shared),
        FriendlyName("Display Fields"),
        Description("Specify which fields should be displayed (Field1;Field2)")]
        public string DisplayFields
        {
            get
            {
                if (_DisplayFields != null)
                {
                    return String.Join(";", (string[])_DisplayFields.ToArray(typeof(string)));
                }
                else
                {
                    return "";
                }
            }
            set
            {
                if (!String.IsNullOrEmpty(value))
                {
                    _DisplayFields = new ArrayList();
                    _DisplayFields.AddRange(value.Split(';'));
                }
                else
                {
                    _DisplayFields = null;
                }
            }
        }

        [Browsable(true),
        Category("Properties"),
        WebPartStorage(Storage.Shared),
        FriendlyName("Filter String"),
        Description("a non case-sensitive Filter (Country='DE')")]
        public string FilterString
        {
            get { return _FilterString; }
            set { _FilterString = value; }
        }
        #endregion

        #region ICallbackEventHandler Member
        public string GetCallbackResult()
        {
            return this._AjaxData;
        }

        public void RaiseCallbackEvent(string eventArgument)
        {
            this._AjaxData = CalculateContent();
        }
        #endregion

        protected override void CreateChildControls()
        {
            base.CreateChildControls();

            try
            {
                _Div = new HtmlGenericControl("DIV");
                // display an image, until the content is received and displayed instead
                Image img = new Image();
                img.ImageUrl = "/_layouts/images/CRPERSPC.GIF";//kpiprogressbar.GIF";
                _Div.Controls.Add(img);
                this.Controls.Add(_Div);
            }
            catch (Exception ex)
            {
                _ErrorText += "Error in CreateChildControls:\r\n" + ex.ToString() + "\r\n";
            }
        }

        #region Pre-/Render
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            #region Create Ajax Script
            _DataDiv = _Div.ClientID;

            string js = Page.ClientScript.GetCallbackEventReference(this, "'blah'", "filldiv", "'" + _DataDiv + "'", true);

            string contentloader = "var ajaxcommands='';  window.onload = ajaxloader; function ajaxloader () { eval(ajaxcommands); } function filldiv(arg, ctx) { var mydiv = document.getElementById(ctx); mydiv.innerHTML = arg; }";
            if (Page.ClientScript.IsClientScriptBlockRegistered("contentloader ") == false)
            {
                Page.ClientScript.RegisterClientScriptBlock(Page.ClientScript.GetType(), "contentloader", contentloader, true);
            }
            // use myloader + this.ID to have unique scripts if there are multiple instances of the webpart on a page
            Page.ClientScript.RegisterStartupScript(this.GetType(), "myloader" + this.ID, " ajaxcommands = ajaxcommands + \"" + js + ";\";", true);
            #endregion
        }

        protected override void RenderContents(HtmlTextWriter writer)
        {
            // if there was an error, display it
            if (!String.IsNullOrEmpty(_ErrorText))
            {
                HtmlGenericControl errorControl = new HtmlGenericControl("DIV");
                errorControl.Attributes.Add("style", "padding: 20px; margin: 20px; border-style: groove; background-color: #FFFFFF;");
                errorControl.InnerHtml = "<IMG src=\"/_layouts/images/error16by16.gif\" border=\"0\"/>" + _ErrorText;
                errorControl.RenderControl(writer);
            }

            EnsureChildControls();

            // render everything, we added to the Controls so far
            RenderChildren(writer);
        }
        #endregion

        private string CalculateContent()
        {
            string returnString = string.Empty;

            try
            {
                TextWriter tw = new StringWriter();
                HtmlTextWriter _TxtWriter = new HtmlTextWriter(tw);

                _Grid = new SPGridView();
                _Grid.DataSource = GetGridDataSource();
                _Grid.ID = GridClientID;
                _Grid.AutoGenerateColumns = false;

                // catch if the grid datasource is empty
                if (_DS == null)
                {
                    return "<IMG src=\"/_layouts/images/BROKENLINK.GIF\" border=\"0\"/> no data received";
                }

                GenerateGridColumns(_Grid, _DS);

                #region Grouping
                if (!String.IsNullOrEmpty(_GroupField) && DataSetContainsField(_DS, _GroupField))
                {
                    _Grid.AllowGrouping = true;
                    _Grid.GroupField = _GroupField;
                }
                #endregion
                Sorting();
                Filtering();
                _Grid.DataBind();

                this.Controls.Add(_Grid);
                _Grid.RenderControl(_TxtWriter);
                returnString = _TxtWriter.InnerWriter.ToString();
            }
            catch (Exception ex)
            {
                _ErrorText += "Error in CalculateContent:\r\n:" + ex.ToString() + "\r\n";
                returnString = _ErrorText;
            }

            return returnString;
        }

        #region DataSet
        /// <summary>
        /// Queries the Webservice and writes its result into the ds DataSet
        /// </summary>
        private void GetDataSet()
        {
            Service.Service ws = new SPAjaxDemo.Service.Service();
            ws.Url = "http://urltowebservice/service.asmx";
            ws.Credentials = CredentialCache.DefaultNetworkCredentials;
            _DS = ws.ReadSomething();
        }

        /// <summary>
        /// Gets a DataView from the DS DataSet
        /// </summary>
        /// <returns></returns>
        private DataView GetGridDataSource()
        {
            if (_DS == null)
            {
                GetDataSet();
            }

            if (_DS != null && _DS.Tables.Count > 0 && _DS.Tables[0] != null)
            {
                return _DS.Tables[0].DefaultView;
            }

            return null;
        }

        /// <summary>
        /// Created Grid Columns
        /// </summary>
        /// <param name="grid"></param>
        /// <param name="ds"></param>
        private void GenerateGridColumns(SPGridView grid, DataSet ds)
        {
            if (ds.Tables.Count == 0)
            {
                return;
            }

            if (_DisplayFields == null)
            {
                for (int i = 0; i < ds.Tables[0].Columns.Count; i++)
                {
                    BoundField field = new BoundField();
                    field.DataField = ds.Tables[0].Columns[i].ColumnName;
                    field.HeaderText = ds.Tables[0].Columns[i].ColumnName;
                    field.SortExpression = ds.Tables[0].Columns[i].ColumnName;

                    grid.Columns.Add(field);
                }
            }
            else
            {
                foreach (string fieldName in _DisplayFields)
                {
                    BoundField field = new BoundField();
                    field.DataField = fieldName;
                    field.HeaderText = fieldName;
                    field.SortExpression = fieldName;

                    if (DataSetContainsField(ds, fieldName))
                    {
                        grid.Columns.Add(field);
                    }
                }
            }
        }

        /// <summary>
        /// Check if a DataSet contains a field
        /// </summary>
        /// <param name="ds">DataSet to check</param>
        /// <param name="FieldName"></param>
        /// <returns></returns>
        private bool DataSetContainsField(DataSet ds, string FieldName)
        {
            bool contains = false;

            for (int i = 0; i < ds.Tables[0].Columns.Count; i++)
            {
                if (ds.Tables[0].Columns[i].ColumnName == FieldName)
                {
                    contains = true;
                    return contains;
                }
            }

            return contains;
        }
        #endregion

        #region Events
        /// <summary>
        /// sorts the Grid Datasource
        /// </summary>
        /// <param name="_Grid"></param>
        private void Sorting()
        {
            if (!String.IsNullOrEmpty(_SortString))
            {
                string[] sortArray = _SortString.Split(';');
                if (sortArray.Length == 2)
                {
                    SortExpression = sortArray[0];
                    SortDirection = sortArray[1];
                    if (DataSetContainsField(_DS, SortExpression))
                    {
                        _DS.Tables[0].DefaultView.Sort = SortExpression + " " + SortDirection;
                    }
                }
            }
        }

        /// <summary>
        /// Filters the DataSource 
        /// </summary>
        private void Filtering()
        {
            if (!String.IsNullOrEmpty(_FilterString) && _FilterString.Contains("="))
            {
                string filterColumnName = _FilterString.Substring(0, _FilterString.IndexOf('='));

                // catch typos in the filter
                if (DataSetContainsField(_DS, filterColumnName))
                {
                    _DS.Tables[0].DefaultView.RowFilter = _FilterString;
                }
            }
        }
        #endregion
    }
}
